/*
SqlDemo -  Application shows various functionality of ODBC (Open Database Connectivity).  Specifically,
there are four buttons on the main form/dialog/window which when clicked will do the following:

1) Repeatedly call the ODBC function SqlDrivers() to dump from your registry the installed ODBC drivers
   on your system.  These drivers along with various pertinent information will be displayed in a
   scrollable output screen the program will create.  This information is useful because it is these
   database driver names/strings you need to form an ODBC connection string for connection to the database;

2) Connect to Microsoft Excel Book1.xls file and display contents of file to a scrollable output window
   the program creates.  To exercise this functionality you need to create the Book1.xls file yourself
   because I have not been able to figure out how to use ODBC to create the data -  I only know how to
   read and dump it.  To create the specific data this program was designed to read, open Excel and
   paste the following into the blank Sheet1 of Book1.xls and save this file to the directory from which
   your compiled executable will run.  To correctly past the data in copy this below then place your
   mouse cursor over cell A1 in the Sheet1 of Excel and right click & execute 'Paste'.

   Id	Float_Point	Date_Field	Text_Field
   1	3.14159	11/15/1952	My Birthday
   2	1.23456	6/30/1969	Walk On Moon?
   3	15.1234	1/1/2006	Some String
   4	0.54321	4/1/2006	April Fools Day!

3) Clicking the 3rd button invokes the Microsoft Access database driver and creates an Access database
   named Testdata.mdb in whatever directory you are running the program from.  It then inserts four lines
   of data just as you see above regarding Excel into a Table1 that it also creates within the database.
   Upon successfully completing that task it creates a scrollable output window and dumps the data as well
   as other diagnostic data to there.  I might point out you don't need to have Microsoft Access installed
   for this to work.  I believe a standard Windows install will cause many of these various database drivers
   to be installed on your system.

4) Clicking the 4th button will create an Sql Server Express database exactly similiar to the Access database
   described above if you have an instance of SQL Server Express running on your system.  Sql Server Express
   and Sql Server Management Studio Express can be downloaded for free from Microsoft.  These programs allow
   you to become familiar with these powerful database systems without having to pay very costly fees.

I want to point out one cautionary note!  The way SqlDemo works is that it attempts to read and/or create
databases and files within whatever directory you are running the executable.  There is a major problem
though with several prominant locations on Windows computers; particularly C:\Documents and Settings and
C:\Program Files.  If you attempt to run the executable from any of these locations or folders under these
locations in Windows XP the program will fail to create the databases.  I'm mentioning this, for example,
because the default location Microsoft Visual Studio 2002 on through 2008 will place your various projects
is under your Documents and Settings folder somewhere, and while you'll have no trouble developing or compiling
the program from there, you simply won't be able to run it from there, at least not on XP.  I have no
experience with Vista/Windows 7, etc, with its file virtualization technologies, but I can tell you for a fact
it won't work on XP from those locations.  If you insist on developing the code at that location, you'll have
to copy the executable to somewhere else to run it.  Actually, it'll run from there fine; it just won't create
the databases when you click the Access or Sql Server buttons.  However, in those situations the program will
output diagnostic information to you informing you of various difficulties.

Another thing the program does is create an Output.txt file containing Debug log type information that may
be helpful to you in learning about ODBC or programming in general.  The program will create this Output.txt
file if the symbol MY_DEBUG is defined.  You can find this symbol in Main.h.  Just uncomment the line and the
file will be created.

Concerning the way the program works, when it starts you'll see a small form with four buttons on it.  You can
click the buttons in any sequence and as many times as you like.  Since I couldn't figure out how to do much
with the ODBC drivers for Excel, repeatedly clicking that button won't do a whole lot - it'll just repeatedly
dump those four lines to another output screen.  In the case of the Access and Sql Server buttons though, its a
bit more interesting.

The 1st time you click either of those buttons the program creates the database and Table1 within the database,
then inserts and dumps those four records described above under my discussion of Excel.  Clicking the button
additional times simply adds four more records to the preexisting database, whether it be Access or Sql Server.
So, if you want, you can fill up your screen with output windows.  Each time a button is clicked, memory
allocations are made adequate for storing whatever lines of text are to be displayed in the scrollable output
screen.  Pointers to these memory allocations are stored in the .cbWndExtra bytes of the "frmOutput" class
which is registered just below in fnWndProc_OnCreate().

Another vexing issue I faced in developing this demo is that I had tremendous trouble with the arrow and
hourglass mouse cursors.  I wanted to show an hourglass mouse cursor when processing was taking place and before
the output screen became vusible so as to let the user know something was going on.  However, particularly with
Sql Server, what was happening was that when you clicked the button, you would see either no hourglass at all,
or just a brief flicker of one, and the output screen might not become visible for quite a few seconds
afterwards, which would leave you wandering if the program failed, locked up, or whatever.  My diagnosis of this
issue (and I'm not 100% sure it is right, but I think it is) is that asynchronous processing is taking place
within the various ODBC and driver layers, and this asynchronous processing is taking place much slower than
the Windows GUI code in my app which appears to blow through the various calls to change the cursor to an
hourglass and back to an arrow again.

The only way I was able to solve the problem was to start seperate threads for the various functions containing
ODBC function calls, and this seemed to work.  I realize it made this demo more complex, but I hated the situation
with the cursor turning back to an arrow long before the output screen showed itself.

One final note I'll make is that you will probably find this app to your dislike.  It doesn't use MFC, .NET, or
any other class framework.  In fact, about the only C++ specific thing it does is use my own String class to
lighten the burden of string minipulations.  Its basically a pure Sdk Api style C program except for that and
perhaps my personal inclination to use a for loop instead of a massive switch construct to map Windows messages
to the procedure that will handle them.  In WinTypes.h you'll find my WindowsEventArguements and EVENTHANDLER
structs that I use to 'crack' messages.  With MY_DEBUG turned off the executable is only around 35K with Visual
Studio 2008 and C++.

Oh!  There is another final note actually!  When I started putting together this app I was going to add a button
for Microsoft's MSDE, which was their first incarnation of a stripped down version of SQL Server, which eventually
morphed into SQL Server Express.  While probably not too many folks have this anymore, the coding for it is about
exactly the same as for SQL Server.  Actually, it is SQL Server.  The only difference, if you have this on your
computer, and want to try this program with it, is that instead of this in the connection string for the SERVER
name...

                                     SERVER=YourComputerName\SQLEXPRESS

you'll just use...

                                             SERVER=localhost;

In the procedure SqlServerThread() I have comments that explain how to change it to use MSDE instead of Sql Server
Express.  I expect there still may be a few of these around.  I usually put it on my Windows 2000 laptops yet.

Preprocessor Settings From VC9 - "WIN32;NDEBUG;_WINDOWS;_CRT_SECURE_NO_WARNINGS;_CRT_NON_CONFORMING_SWPRINTFS"
*/
#include      <windows.h>        //Main include file for C++ Windows programming
//#define       _UNICODE         //Needed for Unicode for VC6
#include      <tchar.h>          //Used for portability to ansi & unicode
#include      <stdio.h>          //good old fashioned CRT procedures
#include      <string.h>         //low level C string procedures
#include      <odbcinst.h>       //Necessary to create access database
#include      <sql.h>            //ODBC header file
#include      <sqlext.h>         //ODBC header file
#include      "WinTypes.h"       //Message cracking logic structures
#include      "strings.h"        //prototypes of members of my String class
#include      "Main.h"           //mostly a few equates for the buttons on the main form
#include      "Sql.h"            //Sql Class used to offer ODBC functionality.
#include      "SqlProcs.h"       //Some of my ODBC wrappers to hide some real grungy ODBC code
#include      "frmOutput.h"      //Output screen code
#include      "Excel.h"          //routines for loading & dumping Excel Workbook
#include      "SqlServer.h"      //routines for dealing with SQL Server
#include      "SqlDrivers.h"     //routine to dump to Output Screen the ODBC Drivers on your system
EVENTHANDLER  EventHandler[3];   //struct to associate Windows message with function pointer
EVENTHANDLER  frmOutputHdlr[5];  //which points to event procedure that handles message
#if defined(MY_DEBUG)            //Unremark //#define MY_DEBUG in Main.h to generate Output.txt log file
    FILE*     fp;
#endif


long fnWndProc_OnCreate(lpWndEventArgs Wea)                       //First procedure (other than WinMain()) to
{                                                                 //run in this program.  If MY_DEBUG is defined
 TCHAR szClassName[16];                                           //in Main.h the program attempts to create an
 WNDCLASSEX wc;                                                   //Output.txt output log file in whatever directory
 String s1;                                                       //the program is run from.
 HWND hCtl;                                                       //The next thing the program does after this is
                                                                  //RegisterClassEx() the frmOutput class.  This is
 Wea->hIns = ((LPCREATESTRUCT)Wea->lParam)->hInstance;            //important in this app because it is the output
 #if defined(MY_DEBUG)                                            //screen where data this program creates is
     TCHAR lpBuffer[MAX_PATH];                                    //displayed.  The way this output screen works is
     DWORD nBufLen=MAX_PATH;                                      //it allocates 16 extra .cbWndExtra bytes in its
     GetCurrentDirectory(nBufLen,lpBuffer);                       //WNDCLASSEX struct to hold info about the data it
     s1=lpBuffer;                                                 //will display.  In the various procedures of this
     s1=s1+_T("\\Output.txt");                                    //program that accumulate ODBC data an estimate is
     fp=_tfopen(s1.lpStr(),_T("w"));                              //always made of the number of lines of output that
     _ftprintf(fp,_T("Entering fnWndProc_OnCreate()\n"));         //will be needed.  A memory allocation is made for
     _ftprintf(fp,_T("lpBuffer = %s\n"),lpBuffer);                //a pointer for each line to be displayed.  Further
     _ftprintf(fp,_T("s1.lpStr() = %s\n"),s1.lpStr());            //allocations are made for each needed byte of each
     #if defined(UNICODE)                                         //line.  The pointer to the pointers are stored at
         _ftprintf(fp,TEXT("  UNICODE Is Defined!\r\n"));         //offset four in the cbWndExtra bytes, and the
     #else                                                       //number of pointers (lines) are stored at offset
         _ftprintf(fp,TEXT("  UNICODE Is Not Defined!\r\n"));     //zero.  See the code in frmOutput_OnCreate() to see
     #endif                                                      //details of what's stored there and how it is used.
 #endif

 _tcscpy(szClassName,_T("frmOutput"));                  //frmOutput is our Output Screen          //The output screen
 wc.lpszClassName=szClassName;                          wc.lpfnWndProc=frmOutput;                 //contains scroll logic
 wc.cbSize=sizeof (WNDCLASSEX);                         wc.style=CS_DBLCLKS;                      //that will allow you
 wc.hIcon=LoadIcon(NULL,IDI_APPLICATION);               wc.hInstance=Wea->hIns;                   //to scroll through
 wc.hIconSm=NULL;                                       wc.hCursor=LoadCursor(NULL,IDC_ARROW);    //the data.  The
 wc.hbrBackground=(HBRUSH)GetStockObject(WHITE_BRUSH);  wc.cbWndExtra=16;                         //frmOutput_OnClose()
 wc.lpszMenuName=NULL;                                  wc.cbClsExtra=0;                          //logic de-allocates
 RegisterClassEx(&wc);                                                                            //the memory for the
                                                                                                  //displayed data.
 TCHAR* szCaption[]=
 {
  _T("Display ODBC Drivers On Your Computer"),     //The program's interface consists only of these four buttons on the
  _T("Read Excel Spreadsheet With ODBC"),          //main screen.  You can see what the program does by reading the
  _T("Create, Load And Dump MS Access Database"),  //captions for the buttons just left.  The CreateWindow() calls to
  _T("Create, Load And Dump Sql Server Express")   //create the buttons are just below.
 };
 HINSTANCE hIns=((LPCREATESTRUCT)Wea->lParam)->hInstance;
 DWORD dwStyle=WS_CHILD | WS_VISIBLE;
 hCtl=CreateWindow(_T("button"),szCaption[0],dwStyle,30,20,325,30,Wea->hWnd,(HMENU)(IDC_SQL_DRIVERS),hIns,NULL);
 hCtl=CreateWindow(_T("button"),szCaption[1],dwStyle,30,60,325,30,Wea->hWnd,(HMENU)(IDC_EXCEL),hIns,NULL);
 hCtl=CreateWindow(_T("button"),szCaption[2],dwStyle,30,100,325,30,Wea->hWnd,(HMENU)(IDC_MS_ACCESS),hIns,NULL);
 hCtl=CreateWindow(_T("button"),szCaption[3],dwStyle,30,140,325,30,Wea->hWnd,(HMENU)(IDC_SQL_SERVER_EXPRESS),hIns,NULL);
 #if defined(MY_DEBUG)
     _ftprintf(fp,_T("Leaving fnWndProc_OnCreate()\n\n"));
 #endif

 return 0;
}


long fnWndProc_OnCommand(lpWndEventArgs Wea)
{
 switch(LOWORD(Wea->wParam))
 {
  case IDC_SQL_DRIVERS:
    btnSqlDrivers_OnClick(Wea);        //Found in SqlDrivers.cpp
    break;
  case IDC_EXCEL:
    btnExcel_OnClick(Wea);             //Found In Excel.cpp
    break;
  case IDC_MS_ACCESS:
    btnAccess_OnClick(Wea);            //Found in Access.cpp
    break;
  case IDC_SQL_SERVER_EXPRESS:
    btnSqlServerExpress_OnClick(Wea);  //Found in SqlServer.cpp
    break;
 }

 return 0;
}


long fnWndProc_OnClose(lpWndEventArgs Wea)
{
 HCURSOR hArrow;
 HWND hOutput;

 #if defined(MY_DEBUG)
     _ftprintf(fp,_T("Entering fnWndProc_OnClose()\n"));
 #endif
 hArrow=LoadCursor(NULL,IDC_ARROW);
 SetClassLong(Wea->hWnd, GCL_HCURSOR, (LONG) hArrow);

 //Note!   The code below sends WM_CLOSE messages to any Output windows the user may have failed to [X] out
 //        of before before [X]'ing out of the main window.  Remember, the memory allocations for the lines
 //        of text visible in the various output windows are stored in pointers in the frmOutput Windows
 //        cbWndExtra bytes.  If this memory isn't properly released, it'll leak bad.

 do        //Search And Destroy Mission For Any Sql Drivers Windows Hanging Around
 {
  hOutput=FindWindowEx(0,0,_T("frmOutput"),_T("ODBC Database Drivers On Your System"));
  if(hOutput)
     SendMessage(hOutput,WM_CLOSE,0,0);
  else
     break;
 }while(TRUE);

 do        //Search And Destroy Mission For Any Excel Output Windows Hanging Around
 {
  hOutput=FindWindowEx(0,0,_T("frmOutput"),_T("Data Dump Of Excel Spreadsheet With ODBC"));
  if(hOutput)
     SendMessage(hOutput,WM_CLOSE,0,0);
  else
     break;
 }while(TRUE);

 do        //Search And Destroy Mission For Any Access Output Windows Hanging Around
 {
  hOutput=FindWindowEx(0,0,_T("frmOutput"),_T("Data Dump Of Access Database With ODBC"));
  if(hOutput)
     SendMessage(hOutput,WM_CLOSE,0,0);
  else
     break;
 }while(TRUE);

 do        //Search And Destroy Mission For Any Sql Server Output Windows Hanging Around
 {
  hOutput=FindWindowEx(0,0,_T("frmOutput"),_T("ODBC Demo With Sql Server"));
  if(hOutput)
     SendMessage(hOutput,WM_CLOSE,0,0);
  else
     break;
 }while(TRUE);

 DestroyWindow(Wea->hWnd);
 PostQuitMessage(WM_QUIT);
 #if defined(MY_DEBUG)
     _ftprintf(fp,_T("Leaving fnWndProc_OnClose()\n\n"));
     fclose(fp);
 #endif

 return 0;
}


void AttachEventHandlers(void)      //This procedure is called from WinMain() and associates
{                                   //Windows messages with the procedure to handle them.
 EventHandler[0].Code=WM_CREATE,    EventHandler[0].fnPtr=fnWndProc_OnCreate;
 EventHandler[1].Code=WM_COMMAND,   EventHandler[1].fnPtr=fnWndProc_OnCommand;  //EventHandler is a struct that looks like this...
 EventHandler[2].Code=WM_CLOSE,     EventHandler[2].fnPtr=fnWndProc_OnClose;    //
 frmOutputHdlr[0].Code=WM_CREATE,   frmOutputHdlr[0].fnPtr=frmOutput_OnCreate;  //  struct EVENTHANDLER
 frmOutputHdlr[1].Code=WM_PAINT,    frmOutputHdlr[1].fnPtr=frmOutput_OnPaint;   //  {
 frmOutputHdlr[2].Code=WM_SIZE,     frmOutputHdlr[2].fnPtr=frmOutput_OnSize;    //   unsigned int    Code;   //i.e., WM_CREATE, etc
 frmOutputHdlr[3].Code=WM_VSCROLL,  frmOutputHdlr[3].fnPtr=frmOutput_OnVScroll; //   long            (*fnPtr)(lpWndEventArgs);
 frmOutputHdlr[4].Code=WM_CLOSE,    frmOutputHdlr[4].fnPtr=frmOutput_OnClose;   //  }
}


long __stdcall fnWndProc(HWND hwnd, unsigned int msg, WPARAM wParam,LPARAM lParam)
{
 WndEventArgs Wea;                           //No doubt about it, I'm a minimalist when it comes to Window Procedures.
                                             //I like to make them as small as possible, and don't like big switch
 for(unsigned int i=0;i<3;i++)               //constructs (don't much like useless macros neither, or any other meaning-
 {                                           //less crap).  The way this works is that when Windows calls this program's
     if(EventHandler[i].Code==msg)           //Window Procedure (fnWndProc()) the program iterates through the array of
     {                                       //EVENTHANDLER structs seeking a match between the msg sent to the WndProc
        Wea.hWnd=hwnd, Wea.lParam=lParam, Wea.wParam=wParam;   //and the .Code member of the EventHandler struct.  If a
        return (*EventHandler[i].fnPtr)(&Wea);                 //match is found, the appropriate event handler function
     }                                                         //is called through the function pointer member .fnPtr
 }                                                             //set in AttachEventHandlers().

 return (DefWindowProc(hwnd,msg,wParam,lParam));
}


int __stdcall WinMain(HINSTANCE hIns, HINSTANCE hPrevIns, LPSTR lpszArgument, int iShow)
{
 TCHAR szClassName[]=TEXT("SqlDemo");
 WNDCLASSEX wc;
 MSG messages;
 HWND hWnd;

 AttachEventHandlers();
 wc.lpszClassName=szClassName;                wc.lpfnWndProc=fnWndProc;
 wc.cbSize=sizeof (WNDCLASSEX);               wc.style=CS_DBLCLKS;
 wc.hIcon=LoadIcon(NULL,IDI_APPLICATION);     wc.hInstance=hIns;
 wc.hIconSm=LoadIcon(NULL, IDI_APPLICATION);  wc.hCursor=LoadCursor(NULL,IDC_ARROW);
 wc.hbrBackground=(HBRUSH)COLOR_BTNSHADOW;    wc.cbWndExtra=0;
 wc.lpszMenuName=NULL;                        wc.cbClsExtra=0;
 RegisterClassEx(&wc);
 hWnd=CreateWindow(szClassName,TEXT("ODBC Demo"),WS_OVERLAPPEDWINDOW,150,100,395,220,HWND_DESKTOP,0,hIns,0);
 ShowWindow(hWnd,iShow);
 while(GetMessage(&messages,NULL,0,0))
 {
  TranslateMessage(&messages);
  DispatchMessage(&messages);
 }

 return messages.wParam;
}


//fp=_tfopen(_T("C:\\Code\\CodeBlks\\SqlDemo\\bin\\Release\\Output.txt"),_T("w"));
